pub struct GlesRenderer { /* private fields */ }renderer_gl only.Expand description
A renderer utilizing OpenGL ES
Implementations
Sourceimpl GlesRenderer
impl GlesRenderer
Sourcepub unsafe fn supported_capabilities(
context: &EGLContext,
) -> Result<Vec<Capability>, GlesError>
pub unsafe fn supported_capabilities( context: &EGLContext, ) -> Result<Vec<Capability>, GlesError>
Get the supported Capabilities of the renderer
Safety
This operation will cause undefined behavior if the given EGLContext is active in another thread.
Sourcepub unsafe fn new(context: EGLContext) -> Result<GlesRenderer, GlesError>
pub unsafe fn new(context: EGLContext) -> Result<GlesRenderer, GlesError>
Creates a new OpenGL ES renderer from a given EGLContext
with all supported capabilities.
Safety
This operation will cause undefined behavior if the given EGLContext is active in another thread.
See: with_capabilities for more information
Sourcepub unsafe fn with_capabilities(
context: EGLContext,
capabilities: impl IntoIterator<Item = Capability>,
) -> Result<GlesRenderer, GlesError>
pub unsafe fn with_capabilities( context: EGLContext, capabilities: impl IntoIterator<Item = Capability>, ) -> Result<GlesRenderer, GlesError>
Creates a new OpenGL ES renderer from a given EGLContext
with the specified Capabilities. If a requested Capability is not supported an
error will be returned.
Safety
This operation will cause undefined behavior if the given EGLContext is active in another thread.
Implementation details
- Texture handles created by the resulting renderer are valid for every rendered created with an
EGLContextshared with the given one (seeEGLContext::new_shared) and can be used on any of these renderers. - This renderer has no default framebuffer, use
Bind::bindbefore rendering. - Shm buffers can be released after a successful import, without the texture handle becoming invalid.
- Texture filtering starts with Linear-downscaling and Linear-upscaling.
- If OpenGL ES 3.0 is not available and the underlying
EGLContextis shared, memory textures will insertglFinish-calls into the pipeline. Consider not sharing contexts, if OpenGL ES 3 isn’t available.
Sourcepub fn capabilities(&self) -> &[Capability]
pub fn capabilities(&self) -> &[Capability]
Returns the supported Capabilities of this renderer.
Sourcepub fn is_software(&self) -> bool
pub fn is_software(&self) -> bool
Returns whether the underlying EGLContext is known to be a software renderer.
Sourceimpl GlesRenderer
impl GlesRenderer
Sourcepub fn egl_context(&self) -> &EGLContext
pub fn egl_context(&self) -> &EGLContext
Get access to the underlying EGLContext.
Note: Modifying the context state, might result in rendering issues. The context state is considered an implementation detail and no guarantee is made about what can or cannot be changed. To make sure a certain modification does not interfere with the renderer’s behaviour, check the source.
Sourcepub fn with_context<F, R>(&mut self, func: F) -> Result<R, GlesError>
pub fn with_context<F, R>(&mut self, func: F) -> Result<R, GlesError>
Run custom code in the GL context owned by this renderer.
The OpenGL state of the renderer is considered an implementation detail and no guarantee is made about what can or cannot be changed, as such you should reset everything you change back to its previous value or check the source code of the version of Smithay you are using to ensure your changes don’t interfere with the renderer’s behavior. Doing otherwise can lead to rendering errors while using other functions of this renderer.
Sourcepub fn with_profiled_context<F, R>(
&mut self,
location: SpanLocation,
func: F,
) -> Result<R, GlesError>
pub fn with_profiled_context<F, R>( &mut self, location: SpanLocation, func: F, ) -> Result<R, GlesError>
Run custom code in the GL context with GPU profiling.
Sets up a GPU profiling span, calls with_context(), then finishes
the span, calls glFlush() and synchronizes the CPU/GPU timestamp.
Sourcepub fn compile_custom_pixel_shader(
&mut self,
src: impl AsRef<str>,
additional_uniforms: &[UniformName<'_>],
) -> Result<GlesPixelProgram, GlesError>
pub fn compile_custom_pixel_shader( &mut self, src: impl AsRef<str>, additional_uniforms: &[UniformName<'_>], ) -> Result<GlesPixelProgram, GlesError>
Compile a custom pixel shader for rendering with GlesFrame::render_pixel_shader_to.
Pixel shaders can be used for completely shader-driven drawing into a given region.
They need to handle the following #define variants:
DEBUG_FLAGSsee below
They receive the following variables:
- varying v_coords
vec2- contains the position from the vertex shader - uniform size
vec2- size of the viewport in pixels - uniform alpha
float- for the alpha value passed by the renderer - uniform tint
float- for the tint passed by the renderer (either 0.0 or 1.0) - only ifDEBUG_FLAGSwas defined
Additional uniform values can be defined by passing UniformNames to the additional_uniforms argument
and can then be set in functions utilizing GlesPixelProgram (like GlesFrame::render_pixel_shader_to).
The shader must not contain a #version directive. It will be interpreted as version 100.
Panics
Panics if any of the names of the passed additional uniforms contains a \0/NUL-byte.
Sourcepub fn compile_custom_texture_shader(
&mut self,
shader: impl AsRef<str>,
additional_uniforms: &[UniformName<'_>],
) -> Result<GlesTexProgram, GlesError>
pub fn compile_custom_texture_shader( &mut self, shader: impl AsRef<str>, additional_uniforms: &[UniformName<'_>], ) -> Result<GlesTexProgram, GlesError>
Compile a custom texture shader for rendering with GlesFrame::render_texture or GlesFrame::render_texture_from_to.
They need to handle the following #define variants:
EXTERNALuses samplerExternalOES instead of sampler2D, requires the GL_OES_EGL_image_external extensionNO_ALPHAneeds to ignore the alpha channel of the texture and replace it with 1.0DEBUG_FLAGSsee below
They receive the following variables:
- varying v_coords
vec2- contains the position from the vertex shader - uniform tex
sample2d- texture sampler - uniform alpha
float- for the alpha value passed by the renderer - uniform tint
float- for the tint passed by the renderer (either 0.0 or 1.0) - only ifDEBUG_FLAGSwas defined
Additional uniform values can be defined by passing UniformNames to the additional_uniforms argument
and can then be set in functions utilizing GlesTexProgram (like GlesFrame::render_texture or GlesFrame::render_texture_from_to).
The shader must contain a line only containing //_DEFINES. It will be replaced by the renderer with corresponding #define directives.
Panics
Panics if any of the names of the passed additional uniforms contains a \0/NUL-byte.